

CREATE OR REPLACE FUNCTION fStockDate(
	_Tipe		INT,
	_Date		DATE,
	_WhID 		VARCHAR, 
	_ItGrpID 	VARCHAR, 
	_BrandID 	VARCHAR, 
	_BPID		VARCHAR, 
	_ModelID	VARCHAR,
	_NotZero	BOOLEAN,
	_BigThanZero	BOOLEAN,
	_active		BOOLEAN
) RETURNS SETOF tStockDate AS
$BODY$
DECLARE
	_retval		tStockDate%ROWTYPE;

	_PID		VARCHAR(25); 
	_Qty		NUMERIC(15,4); 
	_QtyStock	NUMERIC(15,4);
	_Stock		VARCHAR; 
	_ItGrpName	VARCHAR(50); 
	_WhName		VARCHAR(50); 
	_ItemDesc	VARCHAR(50); 
	_ItemID		VARCHAR(20); 
	_BrandName	VARCHAR(50); 
	_UnitDesc	VARCHAR(25); 
	_ModelName	VARCHAR(50);
	_Subtotal	NUMERIC(19,4);

	_UsePID		BOOLEAN;

	cStock		REFCURSOR;

	__Date		DATE;

	_PYear		INT2;
	_Day		INT;
	_Month		INT;
	_StartDate	DATE;

	__Qty		NUMERIC(15,4);
	__QtyBal	NUMERIC(15,4);
BEGIN
	IF (_Tipe=0 OR _Date IS NULL) THEN
		__Date:=CURRENT_DATE;
	ELSE
		__Date:=_Date;
	END IF;

	_PYear:=EXTRACT(year FROM __Date);
	_Month:=EXTRACT(month FROM __Date);
	_Day:=EXTRACT(day FROM __Date);
	_StartDate:=TO_DATE((_PYear||'-'||_Month||'-01'), 'YYYY-MM-DD');
	_Month:=_Month-1;

	OPEN cStock FOR
		SELECT vw.itemid, i.itemdesc, 0 AS qty, it.itgrpname, 
		b.brandname AS brandid, i.unitdesc, m.modelname 
		FROM (SELECT DISTINCT itemid 
			FROM stock 
			WHERE (itemid<>'0' AND itemid<>'-1') AND (_WhID IS NULL OR whid=_WhID)
		) vw
		JOIN item i ON vw.itemid=i.itemid
		LEFT JOIN brand b ON i.brandid=b.brandid
		LEFT JOIN model m ON i.modelid=m.modelid
		LEFT JOIN itgrp it ON i.itgrpid=it.itgrpid
		WHERE (_BPID IS NULL OR i.vendorid=_BPID)
		AND (_ItGrpID IS NULL OR i.itgrpid IN (SELECT * FROM fItGrpList(_ItGrpID))) 
		AND (_BrandID IS NULL OR i.brandid=_BrandID)
		AND (_ModelID IS NULL OR i.modelid=_ModelID)
		ORDER BY it.itgrpname, vw.itemid; 
	LOOP
		FETCH cStock INTO _ItemID, _ItemDesc, _Qty, _ItGrpName, _BrandName, _UnitDesc, _ModelName;
		EXIT WHEN NOT FOUND;

		SELECT INTO __QtyBal * FROM fStockDate_Qty(_ItemID, _Month, _PYear, _WhID);
		_Qty:=_Qty+COALESCE(__QtyBal, 0);

		SELECT INTO __Qty SUM(fQtyZToQty1(s.itemid, s.qty, s.unit)) AS qty 
		FROM stockad s 
		JOIN stocka m ON s.stockano=m.stockano
		WHERE s.itemid=_ItemID AND (_WhID IS NULL OR s.whid=_WhID) 
		AND m.stockadate>=_StartDate AND m.stockadate<=__Date;

		IF NOT FOUND THEN
			__Qty:=0;
		END IF;

/* Dimatikan kalau ngambil dari stockad bila ada penyesuaian hpp, akhirnya salah, ngambil dari stocklog aja*/
/*		SELECT INTO _Subtotal SUM(s.subtotal) 
		FROM stockad s 
		JOIN stocka m ON s.stockano=m.stockano
		WHERE s.itemid=_ItemID AND (_WhID IS NULL OR s.whid=_WhID) 
		AND m.stockadate<=__Date;*/

		SELECT INTO _UsePID usepid FROM item WHERE itemid=_ItemID;

		IF(_UsePID = TRUE) THEN
			SELECT INTO _Subtotal SUM(s.subtotal) 
			FROM stockad s 
			JOIN stocka m ON s.stockano=m.stockano
			WHERE s.itemid=_ItemID AND (_WhID IS NULL OR s.whid=_WhID) 
			AND m.stockadate<=__Date;
		ELSE
			SELECT INTO _Subtotal subtotalnew 
			FROM stocklog 
			WHERE itemid=_ItemID AND (_WhID IS NULL OR whid=_WhID) 
			AND stocklogdate<=__DATE ORDER BY stocklogdate DESC,stocklogid DESC LIMIT 1 ;
		END IF;

		_Subtotal:=COALESCE(_Subtotal, 0);

		_Qty:=_Qty+COALESCE(__Qty,0);

		_Stock:=fQty1Desc(_ItemID, _Qty);

		IF (((_NotZero=TRUE AND _Qty!=0) OR (_NotZero=FALSE)) AND ((_BigThanZero=TRUE AND _Qty>0) OR (_BigThanZero=FALSE))) THEN 
			_retval.itemid:=_ItemID;
			_retval.itemdesc:=_ItemDesc;
			_retval.qty:=_Qty;
			_retval.stock:=_Stock;
			_retval.itgrpname:=_ItGrpName;
			_retval.brandname:=_BrandName;
			_retval.unitdesc:=_UnitDesc;
			_retval.modelname:=_ModelName;
			_retval.subtotal:=_Subtotal;

			RETURN NEXT _retval;
		END IF;
	END LOOP;
	CLOSE cStock;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fStockDate(
	_Tipe		INT,
	_Date		DATE,
	_WhID 		VARCHAR, 
	_ItGrpID 	VARCHAR, 
	_BrandID 	VARCHAR, 
	_BPID		VARCHAR, 
	_ModelID	VARCHAR,
	_NotZero	BOOLEAN,
	_BigThanZero	BOOLEAN
) RETURNS SETOF tStockDate AS
$BODY$
DECLARE
	_retval		tStockDate%ROWTYPE;
	cStock		REFCURSOR;
BEGIN
	OPEN cStock FOR 
		SELECT * FROM fStockDate(_Tipe, _Date, _WhID, _ItGrpID, _BrandID, _BPID, _ModelID, _NotZero, _BigThanZero, TRUE);
	LOOP
		FETCH cStock INTO _retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cStock;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spCashA_Reconcile (
	__CashANo		VARCHAR,
	_isReconciled		BOOLEAN,
	_ReconciledBy		VARCHAR,
	__ReconcileNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_CashID			VARCHAR;
	_CashADate		DATE;
	_Amount			NUMERIC(19,4);

	_TotalAsli		NUMERIC(19,4);
	_Total			NUMERIC(19,4);

	arrayCashA		VARCHAR[];
	_CashANo		VARCHAR;
	_arrayIndex		INT2;

	arrayReconNo		VARCHAR[];
	_ReconcileNo		VARCHAR;
	_arrayIndex2		INT2;


	_AccCash		VARCHAR(10);
	_AccUnreconciled	VARCHAR(10);
	_AccNo			VARCHAR(10);
	_Count			INT;
	_JnlNo			VARCHAR(15);
	_JnlNote		VARCHAR;
	_isAutoGen		BOOLEAN;
	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
	_DeptID			VARCHAR(5);
	_PrjID			VARCHAR(5);
	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;
	_RefNo			VARCHAR;
	_RefType		VARCHAR;
	_CrcID			VARCHAR;
	_ExcRate		NUMERIC(19,4);
	_FisRate		NUMERIC(19,4);

	_InOut			VARCHAR(1);
	_DK_CashBank		VARCHAR(1);
	_DK_Unrec		VARCHAR(1);
BEGIN
	/** inisialisasi variabel */
	_CashANo:=NULL;
	_Amount:=0;
	_Total:=0;
	_TotalAsli:=0;
	_CrtBy:=_ReconciledBy;
	_CrtDate:=CURRENT_DATE;
	_UpdBy:=_ReconciledBy;
	_UpdDate:=CURRENT_DATE;
	_RefNo:=NULL;

	arrayCashA:=STRING_TO_ARRAY(__CashANo, ',');
	_arrayIndex:=1;	

	arrayReconNo:=STRING_TO_ARRAY(__ReconcileNo, ',');
	_arrayIndex2:=1;

	LOOP
		_CashANo:=arrayCashA[_arrayIndex];
		_ReconcileNo:=arrayReconNo[_arrayIndex2];
		EXIT WHEN _CashANo IS NULL;

		SELECT INTO _CashID, _CashADate, _Amount, _ExcRate, _CrcID, _FisRate, _RefNo, _RefType 
			cashid, cashadate, amount, excRate, crcID, fisrate, refno, reftype  
		FROM casha WHERE cashano=_CashANo;

RAISE NOTICE 'Processing CashA Reconcile % : % : %', _CashANo, _Amount, _ExcRate;

		IF (_isReconciled=TRUE) THEN
			SELECT INTO _spret * FROM spCashBal_Update(_CashID, _CashADate, _Amount);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_TotalAsli:=_TotalAsli+_Amount;
			_Total:=_Total+(_Amount*_ExcRate);
		ELSE
			SELECT INTO _spret * FROM spCashBal_Update(_CashID, _CashADate, _Amount*(-1));
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_TotalAsli:=_TotalAsli+_Amount;
			_Total:=_Total+(_Amount*_ExcRate);
			
		END IF;

		_RefNo:=COALESCE(_RefNo, _CashANo);
		_RefType:=COALESCE(_RefType, '');

		UPDATE CashA SET isreconciled=_isReconciled,reconcileby=_UpdBy,reconciledate=_UpdDate WHERE cashano=_CashANo;

		_arrayIndex:=_arrayIndex+1;
		_arrayIndex2:=_arrayIndex2+1;
	END LOOP;

/** //////////////////////////// AKUNTANSI //////////////////////////////////// **/
	IF isAccInstalled() THEN
		_AccUnreconciled:=fAccDefault('UNRECBANK');
		IF (_AccUnreconciled IS NULL OR _AccUnreconciled='') THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Akun Default Untuk unreconciled bank belum diset (spCashA_Reconcile)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF (_isReconciled=TRUE) THEN
			_arrayIndex:=1;	
			LOOP
				_CashANo:=arrayCashA[_arrayIndex];
				EXIT WHEN _CashANo IS NULL;

				SELECT INTO 
					_CashID, _CashADate, _Amount, _DeptID, _PrjID, _ExcRate, _InOut, _RefNo, _RefType, _PerID, _BranchID, _CrcID 
					cashid, cashadate, amount, deptid, prjid, ExcRate, InOut, RefNo, RefType, PerID, BranchID, CrcID  
				FROM CashA WHERE cashano=_CashANo;
				_AccCash:=fAccCash(_CashID);

				_RefNo:=COALESCE(_RefNo, _CashANo);
				_RefType:=COALESCE(_RefType, '<kosong>');
		
				_Count:=0;
				_JnlNo:=fTransNoGet('JURNAL');
				_JnlNote:= 'Rekonsiliasi Bank '||':'||CURRENT_DATE;
				SELECT INTO _spret * FROM spJnl_Insert(
					_JnlNo, _RefType, _CashADate, _RefType, _RefNo,
					_JnlNote,true,true,cast(0 AS NUMERIC(19,4)),
					_CrtBy,_CrtDate,_UpdBy,_UpdDate,
					_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
				);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				_Count:=_Count+1;
				IF (_InOut='I') THEN
					_DK_CashBank:='D';
					_DK_Unrec:='K';
				ELSE
					_DK_CashBank:='K';
					_DK_Unrec:='D';
				END IF;
RAISE NOTICE 'Processing Jurnal Reconcile % : % : %', _CashANo, _Amount, _ExcRate;
					--- Ayat Cash/Bank ----
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccCash, _DK_CashBank, ABS(_Amount)*_ExcRate, _DeptID, _PrjID, ABS(_Amount));
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				--- Ayat Unreconciled-Bank  
				_Count:=_Count+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccUnreconciled, _DK_Unrec, ABS(_Amount)*_ExcRate, _DeptID, _PrjID, ABS(_Amount));
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				_arrayIndex:=_arrayIndex+1;

				SELECT INTO _spret * FROM spJnl_New(_JnlNo);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END LOOP;
		ELSE
			-- Void Jurnal Reconcile yang ada

		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spCashA_Reconcile (
	__CashANo	VARCHAR,
	_isReconciled	BOOLEAN,
	_ReconciledBy	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
BEGIN
	/** Inisialisasi */

	SELECT INTO _spret * FROM spCashA_Reconcile(__CashANo, _isReconciled, _ReconciledBy, NULL);
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


